#!/usr/bin/perl -w
use strict;
use warnings;
use Tk;
use Sys::Hostname;

my $host = hostname;
my $logpath = "/usr2/log";



my $progname = "Dynamic Scheduling Big Red Button";
my $progvers = "0.0";

############### Configuration ######################

#location and name of the override file
my $override_file_directory="/home/oper/dysched";
# Read this file. If it contains the text "override=true" dysched.pl will not adjust the schedule, assuming
# someone has locked off the antenna
my $override_file="override.txt";

#Station ID
my $station_id = "hb";

# Schedule/status server address
my $server = "fourier.phys.utas.edu.au";
# Schedule/status server port
my $server_port = 63000;

############### End of Configuration section ######################



# GUI widget thinggies
my $font = '*-helvetica-medium-r-*-*-12-*';
my $boldfont = '*-helvetica-bold-r-*-*-12-*';
my $large_font = '*-helvetica-medium-r-*-*-34-*';
my $large_bold_font = '*-helvetica-bold-r-*-*-34-*';
my $medium_font = '*-helvetica-medium-r-*-*-20-*';
my $medium_bold_font = '*-helvetica-bold-r-*-*-20-*';
my $small_font = '*-helvetica-*-*-*-*-10-*';

my $main;
my ($debug, $log_t, $status_t, $big_button, $button_frame);

my $override = 0;

my $button_label = "Disable Station";


# open a log file
my $logfile = logname($logpath."/ds_allow_".$station_id);



init_gui();
open (my $fh, ">$logfile") or die "Can't open log file: $!\n";
logit($fh, "Opened log file $logfile on $host\n");
#$main->repeat(2000, \&loop_through);

MainLoop();


sub loop_through {
}




#----------------------------------------------------------------------------- 



sub init_gui {
  if ($debug) {print "Starting GUI...\n";}
  $main = MainWindow->new;
  $main->title("Dynamic Scheduling Big Red Button");
  $main->iconname("Dynamic Scheduling Big Red Button");
  
  $main->protocol('WM_DELETE_WINDOW' => \&exit_program);

  $main->fontCreate('bigfont', -family => 'OpenSymbol', -size => 25);
  $main->fontCreate('bigboldfont', -family => 'Symbol', -size => 25, -weight => 'bold', -underline => 1);
  $large_font = 'bigfont';
  $large_bold_font = 'bigboldfont';
  $main->fontCreate('medfont', -family => 'OpenSymbol', -size => 15);
  $main->fontCreate('medboldfont', -family => 'Ariel', -size => 15, -weight => 'bold', -underline => 1);
  $medium_font = 'medfont';
  $medium_bold_font = 'medboldfont';
  $main->fontCreate('regfont', -family => 'OpenSymbol', -size => 8);
  $main->fontCreate('regboldfont', -family => 'OpenSymbol', -size => 8, -weight => 'bold', -underline => 1);
  $font = 'regfont';
  
  $boldfont = 'regboldfont';
  # background colors for variables
  my $bg1 = '#ebddba';
  my $bg2 = '#f5ebdd';

  # background colors for labels
  my $bgl1 = '#c1c1c1';
  my $bgl2 = '#d0d0d0';

  $button_frame = $main->Frame(-relief => 'groove', 
			       -borderwidth => 3)
    ->grid(-row => 2, -column => 0, -columnspan =>4, -sticky => 'ew');
  
#  $big_button = $button_frame->Button(-textvariable    => \$button_label, -font => 'bigboldfont',
#			-justify => 'left',
#			-height => 2,
#			-width => 20,
#			-borderwidth => 8,
#			-pady => 10,
#			-command => \&toggle_override_status)
#   ->grid(-row => 1, -column => 1, -padx => 2);

  $button_frame->Radiobutton(-text => "Enable", -variable    => \$override, -value => 0,
#			     -font => 'boldfont',
			     -justify => 'left',
#			     -height => 2,
#			     -width => 20,
#			     -borderwidth => 8,
#			     -pady => 10
			    )
  ->grid(-row => 1, -column => 2, -padx => 2);

    $button_frame->Radiobutton(-text => "Disable", -variable    => \$override, -value => 1,
#			     -font => 'boldfont',
			     -justify => 'left',
			       #			     -height => 2,
			       #			     -width => 20,
			       #			     -borderwidth => 8,
			       #			     -pady => 10
			      )
  ->grid(-row => 1, -column => 3, -padx => 2);

  $big_button = $button_frame->Button(-text    => "Apply",
#				      -font => 'boldfont',
			-justify => 'left',
#			-height => 2,
#			-width => 20,
#			-borderwidth => 8,
#			-pady => 10,
			-command => \&toggle_override_status)
    ->grid(-row => 1, -column => 4, -padx => 2);

  my $log_frame = $main->Frame(-relief => 'groove', 
			       -borderwidth => 0)
    ->grid(-row => 5, -column => 0, -columnspan =>4, -sticky => 'ew');

  # log window goes here
  #  $log_frame->Label(-relief => 'flat',
  #		    -text => "Messages :", -font => $boldfont)
  #   ->pack(-anchor => 'w');
  
  $log_t = $log_frame->Scrolled(qw/Text -relief sunken 
				   -borderwidth 2 -setgrid true -width 90
				   -height 8 -background white -scrollbars e/)
    ->pack(-expand => 1, -fill =>  'both');
  $log_t->mark(qw/set insert 0.0/);
  $log_t->tagConfigure('r', 'foreground', 'red');
  $log_t->tagConfigure('g', 'foreground', 'green');
  $log_t->tagConfigure('b', 'foreground', 'blue');
  $log_t->tagConfigure('y', 'foreground', 'blue', 'background', 'yellow');
  $log_t->insert('0.0',"$progname version $progvers.\n",'y');

}



sub pcfs_date {
  #declare local to aviod clobbering global variables
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst);
  
  ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)= gmtime(time);
  $yday++; # PERL starts the julian day at zero
  return sprintf("%4.4d.%3.3d.%2.2d:%2.2d:%2.2d", $year+1900,$yday,$hour,$min,$sec);
}

sub logname {
  # returns a string containing a log file name. Input is a prefix, output is that plus a string with the date
  my ($prefix) = @_;
  #declare local to aviod clobbering global variables

  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst);
  
  ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)= gmtime(time);
  $yday++; # PERL starts the julian day at zero
  return sprintf($prefix."_%4.4d_%3.3d_%2.2d%2.2d%2.2d.log", $year+1900,$yday,$hour,$min,$sec);
}

sub logit {
  my ($fh, $text) = @_;
#  inject_snap($text);
  $text =~ s/\\\"/\"/;
  $text = &pcfs_date." ".$text;
#  print LOG $text;
  logmsg($fh, $text);
  if ($debug) {print "$text";}
}


sub logmsg {
  my ($fh, $text) = @_;
  if (defined $log_t) {
    $log_t->insert('end',$text);
    $log_t->yview(moveto => 1);
    $log_t->update;
  }
  print $fh $text;
}

sub logit_red {
  my ($fh, $text) = @_;
  $text = &pcfs_date." ".$text;
#  print LOG $text;
  logmsg_red($fh, $text);
  if ($debug) {print "$text";}
}

sub logit_blue {
  my ($fh, $text) = @_;
  $text = &pcfs_date." ".$text;
#  print LOG $text;
  logmsg_blue($fh, $text);
  if ($debug) {print "$text";}
}

sub logit_green {
  my ($fh, $text) = @_;
  $text = &pcfs_date." ".$text;
#  print LOG $text;
  logmsg_green($fh, $text);
  if ($debug) {print "$text";}
}
sub logmsg_red {
  my ($fh, $text) = @_;
  $log_t->insert('end',$text,'r');
  $log_t->yview(moveto => 1);
  $log_t->update;
  print $fh  $text;
}

sub logmsg_green {
  my ($fh, $text) = @_;
  $log_t->insert('end',$text,'g');
  $log_t->yview(moveto => 1);
  $log_t->update;
  print $fh $text;
}

sub logmsg_blue {
  my ($fh, $text) = @_;
  $log_t->insert('end',$text,'b');
  $log_t->yview(moveto => 1);
  $log_t->update;
  print $fh, $text;
}



sub exit_program {

  exit;
}

sub toggle_override_status {
  my $exec;
  if ($override) {
    logit($fh,"Antenna will be taken out of Dynamic Scheduling\n");
    $exec = "echo \"AVAILABLE $station_id 0\" | nc -w 1 $server $server_port";
    $button_frame->configure(-background => 'orangered');
  } else {
    logit($fh,"Antenna will be included in Dynamic Scheduling\n");
    $exec = "echo \"AVAILABLE $station_id 1\" | nc -w 1 $server $server_port";
    $button_frame->configure(-background => 'palegreen');
  }

  # Update the override file which dysched checks
  open(STATFILE, ">$override_file_directory/$override_file") || die "Can't open $override_file: $!\n";
  print STATFILE "# if override=true, the dynamic scheduling software will assum the telescope is unavailable\n";
  print STATFILE "# and will not change the schedule\n";
  if ($override) {
    print STATFILE "override=true\n";
  } else {
    print STATFILE "override=false\n";
  }
  print STATFILE "#\n";
  close STATFILE;

  my $error = 0;
  open (TEST, "$exec 2>&1 1>/dev/null |") || die "Failed to execute $exec: $!\n";
    while ( <TEST> ) {
      chomp;
#      print "line found : $_\n";
      logit_red($fh, "ERROR: $_\n");
      $error = 1;
      $button_frame->configure(-background => 'lightgrey');
    } 
  close TEST;

  if ($error) {
    logit($fh,"Command to change antenna state failed\n");
    logit($fh,"Probably no change in state\n");
  }  
}

sub uc2lc {
# converts upper case to lower case
# call $lc_string = uc2lc($string);
    my ($lc_string) = @_;
    $lc_string = "\L$_[0]\E";
    return $lc_string;
}
